// Pulser -- extends Flasher by gradually increasing brightness then back down again
// initial version using cosine function.

#ifndef _Pulser_h_
#define _Pulser_h_

#include "Flasher.h"

#define PULSER_VERSION "$Id: Pulser.h 9 2018-10-01 07:19:06Z cafelizardo $"

class Pulser : public Flasher
{
    long interval;
    long offset;        // offset to be applied after computing value to force clipping either low or high
    long increment;     // increment on step_angle
    long maxValue;      // max brightness
    int step_angle;

    public:
    // constructor -- creates a Pulser object
    Pulser() : Flasher() {}
    Pulser(int pin, long interval = 0, long maxValue = 255, long initial = 0, long increment = 1, long offset = 0)
        : Flasher(pin, interval, offset)
    {
        this->interval = interval;
        this->offset = offset;
        this->increment = max(1, increment);
        this->maxValue = maxValue;
        this->step_angle = initial;
    } // ctor

    // modifiers
    Pulser& setInterval(long interval) {
        this->interval = interval;
        return *this;
    }
    Pulser& setMaxValue(long maxValue) {
        this->maxValue = maxValue;
        return *this;
    }
    Pulser& setInitial(long initial) {
        this->step_angle = initial;
        return *this;
    }
    Pulser& setIncrement(long increment) {
        this->increment = increment;
        return *this;
    }
    Pulser& setOffset(long offset) {
        this->offset = offset;
        return *this;
    }

    virtual Pulser& output(int value)
    {
        //Serial.print("Pulser.output "); Serial.print(this->ledPin, DEC); Serial.print(" "); Serial.print(value, HEX); Serial.println("");
        if (this->ledState == DISABLED) {
            digitalWrite(this->ledPin, this->inverted ? HIGH : LOW);
        } else {
            analogWrite(this->ledPin, constrain(this->inverted ? 255 - value : value, 0, 255) );
        }
        return *this;
    }
    virtual Pulser& output()
    {
        //Serial.print("Pulser.output "); Serial.print(this->ledPin, DEC); Serial.print(" ledState="); Serial.print(this->ledState, DEC); Serial.println("");
        this->output(this->ledState);
        return *this;
    }

    // update -- uses sin function to slowly ramp up and down
    virtual Pulser& update()
    {
        if (this->ledState == DISABLED) return *this;

        //Serial.print("pulser update "); Serial.print(this->ledPin, DEC); Serial.print(" ledState="); Serial.print(this->ledState, DEC); Serial.println("");
        // get current time
        unsigned long currentMillis = millis();

        if ( currentMillis - this->previousMillis >= this->interval )
        {
            //convert 0-360 angle to radian (needed for cos function)
            float rad = DEG_TO_RAD * this->step_angle + this->offTime;

            //calculate cosine of angle as number between 0 and maxvalue
            this->ledState = constrain((-cos(rad) * this->maxValue + this->maxValue) + this->offset, 0, this->maxValue);

            this->step_angle += this->increment;
            while (this->step_angle > 360) step_angle -= 360;
            this->previousMillis = currentMillis;
        }
        this->output();
        return *this;
    } // update

    virtual int step(const int dir = 1)
    {
        this->ledState += dir;
        if (this->ledState > this->maxValue) this->ledState = this->maxValue;
        if (this->ledState < 0) this->ledState = 0;
        this->output();
        return this->ledState;
    } // step

}; // class Pulser

#endif // _Pulser_h
